#include <stdio.h>
#include <windows.h>
#include "..\common\CSerial.h"
#include "..\common\CSocket.h"
#include "..\common\LE9x0Command.h"

int main()
{
	int ret = 0;

	// -------------------------------------------------
	// Select the communication method with LE-930R.
	// For the USB interface, open the virtual COM port.
	// For the WiFi interfaces, open a TCP socket.
	// -------------------------------------------------
#if 1
	// Open COM port (COM1)
	CSerial *comm = new CSerial();
	int comPort = 1;
	if( !comm->Open( (void*)&comPort ) )
	{
		ret = -1;
	}
#else
	// Open TCP socket
	// (IP Address = 192.168.0.10, Control port number = 10101)
	CSocket *comm = new CSocket();
	CSocket::CONNECTION connetion = { "192.168.0.10", 10101 };
	if( !comm->Open( (void*)&connetion ))
	{
		ret = -1;
	}
#endif

	if( ret < 0 )
	{
		printf( "Interface open error.\n" );
		goto error_exit;
	}

	// ------------------
	// Connect to LE-940R
	// ------------------
	FRAME Res;
	LE9x0Command *cmd = new LE9x0Command( comm );

	// Invalid connection maintenance command transmission from LE-940R
	cmd->TransmitCommandWait( CMD_CONNECT, 0x20, 0, NULL, &Res );
	if( (Res.res != RES_OK) && (Res.res != RES_ALREADY_CONNECTED) )
	{
		printf( "Connection error. [0x%02X]\n", Res.res );
		ret = -1;
		goto error_comm;
	}
	printf( "Connection OK.\n" );

	// -----------------
	// Setup for LE-940R
	// ------------------
	// Input output type
	char input[64] = "";
	LE940_TYPE type;
	printf( "Enter the output type number.\n" );
	printf( " %d : 32V range\n", LE940_TYPE_VOLTAGE_32V );
	printf( " %d : 4-20mA range\n", LE940_TYPE_CURRENT );
	printf( "> " );
	scanf_s( "%d%*[^\n]", &type );
	if( (type < 0) || (type >= LE940_TYPE_NUM) )
	{
		printf( "The input number is invalid.\n" );
		goto error_cmd;
	}

	float min, max, outIfOpen, outIfShort;
	char* unit;
	switch( type )
	{
	case LE940_TYPE_VOLTAGE_32V:
	case LE940_TYPE_VOLTAGE_32V_1:
		min = -32;
		max = 32;
		unit = "V";
		break;
	case LE940_TYPE_CURRENT:
	case LE940_TYPE_CURRENT_1:
		min = 0;
		max = 20;
		unit = "mA";
		break;
	}

	// Set the output value if DIN is short
	printf( "Set the output value if DIN is short (range of %.0f%s to %.0f%s).\n", min, unit, max, unit );
	printf( "> " );
	scanf_s( "%f%*[^\n]", &outIfShort );
	if( (outIfShort < min) || (outIfShort > max) )
	{
		printf( "Input value is invalid.\n" );
		goto error_cmd;
	}

	// Set the output value if DIN is open
	printf( "Set the output value if DIN is open (range of %.0f%s to %.0f%s).\n", min, unit, max, unit );
	printf( "> " );
	scanf_s( "%f%*[^\n]", &outIfOpen );
	if( (outIfOpen < min) || (outIfOpen > max) )
	{
		printf( "Input value is invalid.\n" );
		goto error_cmd;
	}

	// Acquires the current output value and state of DIN every 100 milliseconds during 10 seconds.
	DWORD start = GetTickCount();
	while (GetTickCount() - start < 10000)
	{
		// Get the current DIN state
		cmd->TransmitCommandWait( CMD_DIN_GET, 0x00, 0, NULL, &Res );
		if ( Res.res != RES_OK )
		{
			printf( "\nFailed to get current state of DIN. [0x%02X]\n", Res.res );
			break;
		}
		printf( "DIN: %s\n", Res.data[0] == 0 ? "off" : "on" );

		// Changes the output value according to the state of DIN
		DWORD out = cmd->ConvertRealDataLE940( type, Res.data[0] == 0 ? outIfOpen : outIfShort );
		BYTE vout[3] = {0};
		vout[0] = (BYTE)type;
		vout[1] = (BYTE)(out >> 8);
		vout[2] = (BYTE)(out & 0xFF);
		cmd->TransmitCommandWait( CMD_DA_OUT_SET, 0x00, sizeof(vout), vout, &Res );
		if( Res.res != RES_OK )
		{
			printf( "Analog output error. [0x%02X]\n", Res.res );
			break;
		}

		Sleep( 100 );
	}

	// Stop output
	BYTE vout[3] = {0};
	vout[0] = (BYTE)type;
	cmd->TransmitCommandWait( CMD_DA_OUT_SET, 0x00, sizeof(vout), vout, &Res );
	if( Res.res != RES_OK )
	{
		printf( "Analog output error. [0x%02X]\n", Res.res );
		goto error_cmd;
	}

error_cmd:
	// ----------------------
	// Disonnect from LE-930R
	// ----------------------
	cmd->TransmitCommandWait( CMD_DISCONNECT, 0x00, 0, NULL, &Res );
	if( Res.res != RES_OK )
	{
		printf( "Disconnection error. [0x%02X]\n", Res.res );
		ret = -1;
		goto error_comm;
	}
	printf( "Disconnection OK.\n" );

error_comm:
	comm->Close();
	delete cmd;

error_exit:
	delete comm;

	return ret;
}
